indexing

	description: "Class that implements whatever an HTTP 1.0 client can do."

	author: "Berend de Boer"
	date: "$Date: 2003/10/02 $"
	revision: "$Revision: #3 $"


class

	EPX_HTTP_10_CLIENT


inherit

	EPX_HTTP_CLIENT

	EPX_MIME_TYPE_NAMES
		export
			{NONE} all;
			{ANY} mime_type_application_x_www_form_urlencoded, mime_type_multipart_form_data
		end

	EPX_MIME_FIELD_NAMES
		export
			{NONE} all;
			{ANY} field_name_content_type
		end

	EPX_HTTP_METHODS
		export
			{NONE} all
		end


creation

	make,
	make_from_port,
	make_from_host,
	make_from_host_and_port


feature -- Client http version

	client_version: STRING is
		once
			Result := "HTTP/1.0"
		end


feature -- Requests

	delete (a_request_uri: STRING) is
		require
			a_request_uri_not_empty: a_request_uri /= Void and then not a_request_uri.is_empty
		do
			send_request (http_method_DELETE, a_request_uri, Void)
		end

	get (a_request_uri: STRING) is
			-- Send GET request to server.
		do
			send_request (http_method_GET, a_request_uri, Void)
		end

	head (a_request_uri: STRING) is
			-- Send HEAD request to server.
			-- `a_request_uri' should not include http: and the host name, only
			-- the page that is requested. Any query and fragment parts are ok.
		require
			a_request_uri_not_empty: a_request_uri /= Void and then not a_request_uri.is_empty
		do
			send_request (http_method_HEAD, a_request_uri, Void)
		end

	options (a_request_uri: STRING) is
			-- Get server options. `a_request_uri' is required when the
			-- request is being made to a proxy.
		do
			if a_request_uri = Void or else a_request_uri.is_empty then
				send_request (http_method_OPTIONS, "*", Void)
			else
				send_request (http_method_OPTIONS, a_request_uri, Void)
			end
		end

	post (a_request_uri: STRING; a_post_data: EPX_MIME_PART) is
		require
			a_request_uri_not_empty: a_request_uri /= Void and then not a_request_uri.is_empty
			post_data_not_void: a_post_data /= Void
			post_data_has_content_type: a_post_data.header.content_type /= Void
			post_data_content_type_recognized:
				a_post_data.header.content_type.value.is_equal (mime_type_multipart_form_data) or else
				a_post_data.header.content_type.value.is_equal (mime_type_application_x_www_form_urlencoded)
			post_data_content_type_has_boundary_if_multipart:
				a_post_data.header.content_type.value.is_equal (mime_type_multipart_form_data) implies
					a_post_data.header.content_type.boundary /= Void and then
					not a_post_data.header.content_type.boundary.is_empty
			post_data_has_body: a_post_data.body /= Void
		do
			send_request (http_method_POST, a_request_uri, a_post_data)
		end


feature {NONE} -- Request implementation

	append_other_fields (request: STRING) is
			-- Append any optional fields
		do
			request.append_string (once_mime_version)
			if accept /= Void then
				request.append_string (field_name_accept)
				request.append_string (once_colon_space)
				request.append_string (accept)
				request.append_string (once_new_line)
			end
			if user_agent /= Void then
				request.append_string (field_name_user_agent)
				request.append_string (once_colon_space)
				request.append_string (user_agent)
				request.append_string (once_new_line)
			end
		end

	send_request (a_request, a_request_uri: STRING; a_request_data: EPX_MIME_PART) is
			-- Send request `a_request' with `a_request_uri' to `host'.
			-- Additional fields and an optional body can be passed in
			-- `a_request_data'.
			-- If sending the request failed (usually because the server refused
			-- the connection), 503 is returned.
		require
			request_not_empty: a_request /= Void and then not a_request.is_empty
			request_uri_not_empty: a_request_uri /= Void and then not a_request_uri.is_empty
		local
			retried: BOOLEAN
			request: STRING
			error_message: STRING
		do
			assert_http_closed
			if not retried then
				response_code := 0
				create request.make (128)
				request.append_string (a_request)
				request.append_character (' ')
				request.append_string (a_request_uri)
				request.append_character (' ')
				request.append_string (client_version)
				request.append_string (once_new_line)
				-- Add required field Host
				request.append_string (field_name_host)
				request.append_string (once_colon_space)
				request.append_string (host.name)
				request.append_string (once_new_line)
				append_other_fields (request)
				request.append_string (once_connection_close)
				if a_request_data = Void then
					request.append_string (once_new_line)
				else
					if a_request_data.header.content_type /= Void and then a_request_data.header.content_type.value.is_equal (mime_type_application_x_www_form_urlencoded) then
						a_request_data.append_urlencoded_to_string (request)
					else
						a_request_data.append_to_string (request)
					end
				end
				create http.open_by_address (sa)
				http.write_string (request)
			else
				response_code := 503
				response_phrase := error_message
			end
		rescue
			if exceptions.is_developer_exception then
				retried := True
				error_message := exceptions.developer_exception_name
				retry
			end
		end


feature -- Fields that are send with a request if set

	accept: STRING
			-- What kind of output can the client handle?
			-- Examples are:
			--   Accept: text/plain; q=0.5, text/html,
			--           text/x-dvi; q=0.8, text/x-c

	user_agent: STRING
			-- Identification of client program.
			-- Common examples are:
			--   Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)
			--   Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.0.0) Gecko/20020529
			--   Microsoft Internet Explorer

	set_accept (value: STRING) is
			-- Set the media types which are acceptable for the response.
		require
			value_void_or_not_empty: value = Void or else not value.is_empty
		do
			accept := value
		ensure
			accept_set:
				(value = Void implies accept = Void) or else
				(accept.is_equal (value))
		end

	set_user_agent (value: STRING) is
			-- Set the client identification.
		require
			value_void_or_not_empty: value = Void or else not value.is_empty
		do
			user_agent := value
		ensure
			user_agent_set:
				(value = Void implies user_agent = Void) or else
				(user_agent.is_equal (value))
		end


feature -- Response

	body: EPX_MIME_BODY_TEXT is
			-- Return body as text, if applicable, else Void.
		do
			Result ?= parser.part.body
		end

	fields: DS_HASH_TABLE [EPX_MIME_FIELD, STRING] is
			-- Header fields of response.
		do
			Result := parser.part.header.fields
		end

	is_response_ok: BOOLEAN is
			-- Does the returned `response_code' indicate success?
		do
			Result := response_code >= 200 and response_code <= 299
		end

	part: EPX_MIME_PART is
			-- The entire parsed MIME message
		do
			Result := parser.part
		end

	read_response is
			-- Read entire resonse, parse while reading.
		do
			-- first line is HTTP version
			if parser = Void then
				create parser.make_from_stream (http)
			else
				parser.make_from_stream (http)
			end
			read_and_parse_status_line
			parser.parse
			if parser.syntax_error then
				response_code := 500
				response_phrase := "Syntax error parsing response."
			end
		end

	response_code: INTEGER

	response_phrase: STRING

	server_version: STRING
			-- Set by `read_response'.


feature -- Individual response fields, Void if not available

	location: STRING is
		do
			fields.search (field_name_location)
			if fields.found then
				Result := fields.found_item.value
			end
		end


feature {NONE} -- Implementation

	read_and_parse_status_line is
			-- First line is HTTP version and response.
			-- But not always, there's some shit out there that
			-- immediately starts with the MIME headers. I don't try to
			-- recover from that.
		local
			p, q: INTEGER
			s: STRING
			status_line: STRING
		do
			create status_line.make (64)
			from
				parser.read_character
			until
				parser.last_character = '%N' or else
				parser.eof
			loop
				status_line.append_character (parser.last_character)
				parser.read_character
			end

			-- Removing %R if part of CRLF
			if status_line.count > 0 and then status_line.item (status_line.count) = '%R' then
				status_line.remove (status_line.count)
			end

			p := status_line.index_of (' ', 1)
			if p > 1 then
				q := status_line.index_of (' ', p+1)
			end
			if p > 1 and q > 1 then
				server_version := status_line.substring (1, p-1)
				sh.trim (server_version)
				s := status_line.substring (p+1, q-1)
				sh.trim (s)
				if s.is_integer then
					response_code := s.to_integer
				else
					response_code := 500
				end
				response_phrase := status_line.substring (q+1, status_line.count)
				sh.trim (response_phrase)
			else
				-- Can we recover from garbage??
				response_code := 500
				response_phrase := ""
				server_version := ""
			end
		end


feature {NONE} -- MIME parser

	parser: EPX_MIME_PARSER
			-- Parse response from HTTP server


feature {NONE} -- Once strings

	once_colon_space: STRING is ": "
	once_connection_close: STRING is "Connection: close%R%N"
	once_mime_version: STRING is "MIME-Version: 1.0%R%N"
	once_new_line: STRING is "%R%N"


end
