indexing

	description: "Headers for a MIME message or MIME part."

	author: "Berend de Boer"
	date: "$Date: 2003/05/08 $"
	revision: "$Revision: #1 $"


class

	EPX_MIME_HEADER


inherit

	ANY

	EPX_MIME_FIELD_NAMES
		export
			{NONE} all
		end


creation

	make_default


feature -- Initialization

	make_default is
			-- Make MIME header with no fields. Useful during parsing to
			-- construct the message gradually.
		do
			create all_fields.make
			create fields.make (64)
		end


feature -- Measurement

	count: INTEGER is
			-- The number of fields in this header.
		do
			Result := all_fields.count
		end


feature -- Status report

	found: BOOLEAN is
			-- Did last call to `search' succeed?
		do
			Result := fields.found
		end

	found_item: EPX_MIME_FIELD is
			-- Item found by last call to `search'
		require
			found: found
		do
			Result := fields.found_item
		end

	has (a_field_name: STRING): BOOLEAN is
			-- Does `a_field_name' exists just once in the header?
		require
			a_field_name_not_empty: a_field_name /= Void and then not a_field_name.is_empty
		do
			Result := fields.has (a_field_name)
		end


feature -- Access

	all_fields: DS_LINKED_LIST [EPX_MIME_FIELD]
			-- All fields.

	fields: DS_HASH_TABLE [EPX_MIME_FIELD, STRING]
			-- All fields that appeared just once in this MIME
			-- message. Add/delete fields by calling `add_field' and
			-- `delete_field'. Never modify this structure directly!

	item (a_field_name: STRING): EPX_MIME_FIELD is
			-- Field corresponding to `a_field_name' if it exists
			-- just once in the header.
		require
			has_item: has (a_field_name)
		do
			Result := fields.item (a_field_name)
		end


feature -- Access to well-known fields

	content_disposition: EPX_MIME_FIELD_CONTENT_DISPOSITION
			-- Field `Content-Disposition' if it exists, else Void.

	content_transfer_encoding: EPX_MIME_FIELD_CONTENT_TRANSFER_ENCODING
			-- Field `Content-Transfer-Encoding' if it exists, else Void.

	content_type: EPX_MIME_FIELD_CONTENT_TYPE
			-- Field `Content-Type' if it exists, else Void.


feature {NONE} -- Set well-known fields

	set_cached_field (a_field: EPX_MIME_FIELD) is
		do
			if a_field.name.is_equal (field_name_content_disposition) then
				content_disposition ?= a_field
			elseif a_field.name.is_equal (field_name_content_transfer_encoding) then
				content_transfer_encoding ?= a_field
			elseif a_field.name.is_equal (field_name_content_type) then
				content_type ?= a_field
			end
		end


feature -- Search

	search (a_field_name: STRING) is
			-- Search if `a_field_name' is a unique field name.
			-- If found, set `found' to true, and set
			-- `found_item' to the found field.
		require
			a_field_name_not_empty: a_field_name /= Void and then not a_field_name.is_empty
		do
			fields.search (a_field_name)
		end


feature -- Modification

	add_field (a_field: EPX_MIME_FIELD) is
			-- Append `a_field' to header. `a_field' must not already exist.
		require
			a_field_not_void: a_field /= Void
			field_not_exists: not has (a_field.name)
		do
			all_fields.put_last (a_field)
			fields.put_last (a_field, a_field.name)
			set_cached_field (a_field)
		ensure
			field_added_to_fields: fields.has (a_field.name)
			field_added_to_all_fields: all_fields.has (a_field)
			field_is_last: all_fields.last = a_field
		end

	add_non_unique_field (a_field: EPX_MIME_FIELD) is
			-- Append `a_field' to header. There may already exist a
			-- field with this name.
		require
			a_field_not_void: a_field /= Void
		do
			all_fields.put_last (a_field)
			fields.put_last (a_field, a_field.name)
			set_cached_field (a_field)
		ensure
			field_added_to_fields: fields.has (a_field.name)
			field_added_to_all_fields: all_fields.has (a_field)
			field_is_last: all_fields.last = a_field
		end

	delete_field (a_field_name: STRING) is
			-- Remove the unique field with name `a_field_name'.
		require
			has_field: has (a_field_name)
		local
			field: EPX_MIME_FIELD
		do
			field := item (a_field_name)
			all_fields.delete (field)
			fields.remove (a_field_name)
			if a_field_name.is_equal (field_name_content_type) then
				content_type := Void
			end
		ensure
			field_removed: not has (a_field_name)
		end

feature -- Output

	append_fields_to_string (s: STRING) is
			-- Stream `fields' to `s' Does not add the body separator.
		require
			s_not_void: s /= Void
		do
			from
				all_fields.start
			until
				all_fields.after
			loop
				all_fields.item_for_iteration.append_to_string (s)
				all_fields.forth
			end
		end


invariant

	fields_not_void: fields /= Void
	content_disposition_type_correct: fields.has (field_name_content_disposition) implies content_disposition /= Void
	content_type_type_correct: fields.has (field_name_content_type) implies content_type /= Void
	content_disposition_set: fields.has (field_name_content_disposition) = (content_disposition /= Void)
	content_transfer_encoding_set: fields.has (field_name_content_transfer_encoding) = (content_transfer_encoding /= Void)
	content_type_set: fields.has (field_name_content_type) = (content_type /= Void)

end
